from typing import TYPE_CHECKING, Any

import msgspec
import yaml

if TYPE_CHECKING:
    from exponent_server.core.tools.edit_tool import (
        EditToolInput as ServerSideEditToolInput,
    )


class PartialToolResult(msgspec.Struct, tag_field="tool_name", omit_defaults=True):
    pass


class ToolInput(msgspec.Struct, tag_field="tool_name", omit_defaults=True):
    """Concrete subclasses describe the full input schema for a tool."""

    def to_llm(self) -> dict[str, Any]:
        """Convert ToolInput to LLM-friendly typed dict format.

        Returns a dictionary with the tool parameters, excluding the tool_name
        which is handled separately by the LLM integration layer.

        Returns:
            self by default, which msgspec will serialize appropriately.
        """
        return msgspec.to_builtins(self)  # type: ignore[no-any-return]

    def to_text(self) -> str:
        d = msgspec.to_builtins(self)
        del d["tool_name"]
        return yaml.dump(d)


class ToolResult(msgspec.Struct, tag_field="tool_name", omit_defaults=True):
    """Concrete subclasses return data from a tool execution."""

    def to_text(self) -> str:
        """
        This provides a default textual representation of the tool result. Override it as needed for your tool."""
        d = msgspec.to_builtins(self)
        del d["tool_name"]
        return yaml.dump(d)


class ErrorToolResult(ToolResult, tag="error"):
    error_message: str


READ_TOOL_NAME = "read"


class ReadToolInput(ToolInput, tag=READ_TOOL_NAME):
    file_path: str
    offset: int | None = None
    limit: int | None = None


class FileMetadata(msgspec.Struct):
    modified_timestamp: float
    file_mode: str


class ReadToolResult(ToolResult, tag=READ_TOOL_NAME):
    content: str
    num_lines: int
    start_line: int
    total_lines: int
    metadata: FileMetadata | None = None

    def to_text(self) -> str:
        lines = self.content.splitlines()
        lines = [
            f"{str(i).rjust(6)}→{line}"
            for i, line in enumerate(lines, start=self.start_line + 1)
        ]
        return "\n".join(lines)


LIST_TOOL_NAME = "ls"


class ListToolInput(ToolInput, tag=LIST_TOOL_NAME):
    path: str
    ignore: list[str] | None = None


class ListToolResult(ToolResult, tag=LIST_TOOL_NAME):
    files: list[str]
    truncated: bool = False


GLOB_TOOL_NAME = "glob"


class GlobToolInput(ToolInput, tag=GLOB_TOOL_NAME):
    pattern: str
    path: str | None = None


class GlobToolResult(ToolResult, tag=GLOB_TOOL_NAME):
    filenames: list[str]
    duration_ms: int
    num_files: int
    truncated: bool


WRITE_TOOL_NAME = "write"


class WriteToolInput(ToolInput, tag=WRITE_TOOL_NAME):
    file_path: str
    content: str


class WriteToolResult(ToolResult, tag=WRITE_TOOL_NAME):
    message: str
    metadata: FileMetadata | None = None


GREP_TOOL_NAME = "grep"


class GrepToolInput(ToolInput, tag=GREP_TOOL_NAME):
    pattern: str
    path: str | None = None
    include: str | None = None
    multiline: bool | None = None


class GrepToolResult(ToolResult, tag=GREP_TOOL_NAME):
    matches: list[str]
    truncated: bool = False


EDIT_TOOL_NAME = "edit"


# This is only used in the CLI. The server side type is edit_tool.py
class EditToolInput(ToolInput, tag=EDIT_TOOL_NAME):
    file_path: str
    old_string: str
    new_string: str
    replace_all: bool = False
    # The last retrieved modified timestamp. This is used to check if the file has been modified since the last read/write that we are aware of (in which case we should re-read the file before editing).
    last_known_modified_timestamp: float | None = None

    @classmethod
    def from_server_side_input(
        cls,
        server_side_input: "ServerSideEditToolInput",
        last_known_modified_timestamp: float | None,
    ) -> "EditToolInput":
        return cls(
            file_path=server_side_input.file_path,
            old_string=server_side_input.old_string,
            new_string=server_side_input.new_string,
            replace_all=server_side_input.replace_all,
            last_known_modified_timestamp=last_known_modified_timestamp,
        )


class EditToolResult(ToolResult, tag=EDIT_TOOL_NAME):
    message: str
    metadata: FileMetadata | None = None


BASH_TOOL_NAME = "bash"


class BashToolInput(ToolInput, tag=BASH_TOOL_NAME):
    command: str
    timeout: int | None = None
    description: str | None = None


class PartialBashToolResult(PartialToolResult, tag=BASH_TOOL_NAME):
    shell_output: str | None = None


class BashToolResult(ToolResult, tag=BASH_TOOL_NAME):
    shell_output: str
    duration_ms: int
    exit_code: int | None
    timed_out: bool
    stopped_by_user: bool


class HttpRequest(msgspec.Struct, tag="http_fetch_cli"):
    url: str
    method: str = "GET"
    headers: dict[str, str] | None = None
    timeout: int | None = None


class HttpResponse(msgspec.Struct, tag="http_fetch_cli"):
    status_code: int | None = None
    content: str | None = None
    error_message: str | None = None
    duration_ms: int | None = None
    headers: dict[str, str] | None = None


ToolInputType = (
    ReadToolInput
    | WriteToolInput
    | ListToolInput
    | GlobToolInput
    | GrepToolInput
    | EditToolInput
    | BashToolInput
)
PartialToolResultType = PartialBashToolResult

ToolResultType = (
    ReadToolResult
    | WriteToolResult
    | ListToolResult
    | GlobToolResult
    | GrepToolResult
    | EditToolResult
    | BashToolResult
    | ErrorToolResult
)


class ToolExecutionRequest(msgspec.Struct, tag="tool_execution"):
    tool_input: ToolInputType


class GetAllFilesRequest(msgspec.Struct, tag="get_all_files"):
    pass


class TerminateRequest(msgspec.Struct, tag="terminate"):
    pass


class SwitchCLIChatRequest(msgspec.Struct, tag="switch_cli_chat"):
    new_chat_uuid: str


# This message is sent periodically from the client to keep the connection alive while the chat is turning.
# This synergizes with CLI-side timeouts to avoid disconnecting during long operations.
class KeepAliveCliChatRequest(msgspec.Struct, tag="keep_alive_cli_chat"):
    pass


class BatchToolExecutionRequest(msgspec.Struct, tag="batch_tool_execution"):
    tool_inputs: list[ToolInputType]


class GetAllFilesResponse(msgspec.Struct, tag="get_all_files"):
    files: list[str]


class TerminateResponse(msgspec.Struct, tag="terminate"):
    pass


class BatchToolExecutionResponse(msgspec.Struct, tag="batch_tool_execution"):
    tool_results: list[ToolResultType]


class SwitchCLIChatResponse(msgspec.Struct, tag="switch_cli_chat"):
    pass


class KeepAliveCliChatResponse(msgspec.Struct, tag="keep_alive_cli_chat"):
    pass


class CliRpcRequest(msgspec.Struct):
    request_id: str
    request: (
        ToolExecutionRequest
        | GetAllFilesRequest
        | TerminateRequest
        | HttpRequest
        | BatchToolExecutionRequest
        | SwitchCLIChatRequest
        | KeepAliveCliChatRequest
    )


class ToolExecutionResponse(msgspec.Struct, tag="tool_execution"):
    tool_result: ToolResultType


class ErrorResponse(msgspec.Struct, tag="error"):
    error_message: str


class CliRpcResponse(msgspec.Struct):
    request_id: str
    response: (
        ToolExecutionResponse
        | GetAllFilesResponse
        | ErrorResponse
        | TerminateResponse
        | BatchToolExecutionResponse
        | HttpResponse
        | SwitchCLIChatResponse
        | KeepAliveCliChatResponse
    )



import asyncio
import sys
import time

import click

from exponent.commands.common import (
    check_inside_git_repo,
    check_running_from_home_directory,
    check_ssl,
    create_chat,
    inside_ssh_session,
    redirect_to_login,
    start_client,
)
from exponent.commands.settings import use_settings
from exponent.commands.types import exponent_cli_group
from exponent.commands.utils import (
    ConnectionTracker,
    Spinner,
    launch_exponent_browser,
    print_exponent_message,
)
from exponent.core.config import Settings
from exponent.core.remote_execution.client import (
    REMOTE_EXECUTION_CLIENT_EXIT_INFO,
    SwitchCLIChat,
    WSDisconnected,
)
from exponent.core.remote_execution.types import ChatSource
from exponent.core.remote_execution.utils import assert_unreachable
from exponent.utils.version import check_exponent_version_and_upgrade

try:
    # this is an optional dependency for python <3.11
    from async_timeout import timeout
except ImportError:  # pragma: no cover
    from asyncio import timeout


@exponent_cli_group()
def run_cli() -> None:
    """Run AI-powered chat sessions."""
    pass


@run_cli.command()
@click.option(
    "--chat-id",
    help="ID of an existing chat session to reconnect",
    required=False,
    prompt_required=False,
    prompt="Enter the chat ID to reconnect to",
)
@click.option(
    "--prompt",
    help="Start a chat with a given prompt.",
)
@click.option(
    "--workflow-id",
    hidden=True,
    required=False,
)
@click.option(
    "--timeout-seconds",
    type=int,
    help="Number of seconds without receiving a request before shutting down",
    envvar="INDENT_TIMEOUT_SECONDS",
)
@use_settings
def run(
    settings: Settings,
    chat_id: str | None = None,
    prompt: str | None = None,
    workflow_id: str | None = None,
    timeout_seconds: int | None = None,
) -> None:
    """[default] Start or reconnect to an Indent session."""
    check_exponent_version_and_upgrade(settings)

    if not settings.api_key:
        redirect_to_login(settings)
        return

    loop = asyncio.get_event_loop()

    check_running_from_home_directory()
    check_ssl()
    loop.run_until_complete(check_inside_git_repo(settings))

    api_key = settings.api_key
    base_url = settings.base_url
    base_api_url = settings.get_base_api_url()
    base_ws_url = settings.get_base_ws_url()

    chat_uuid = chat_id or loop.run_until_complete(
        create_chat(api_key, base_api_url, base_ws_url, ChatSource.CLI_RUN)
    )

    if chat_uuid is None:
        sys.exit(1)

    if (
        not prompt
        and (not inside_ssh_session())
        and (not workflow_id)
        # If the user specified a chat ID, they probably don't want to re-launch the chat
        and (not chat_id)
    ):
        # Open the chat in the browser
        launch_exponent_browser(settings.environment, base_url, chat_uuid)

    while True:
        result = run_chat(
            loop, api_key, chat_uuid, settings, prompt, workflow_id, timeout_seconds
        )
        if result is None or isinstance(result, WSDisconnected):
            # NOTE: None here means that handle_connection_changes exited
            # first. We should likely have a different message for this.
            if result and result.error_message:
                click.secho(f"Error: {result.error_message}", fg="red")
                sys.exit(10)
            else:
                click.echo("Disconnected upon user request, shutting down...")
                break
        elif isinstance(result, SwitchCLIChat):
            chat_uuid = result.new_chat_uuid
            click.echo("\nSwitching chats...")
        else:
            assert_unreachable(result)


def run_chat(
    loop: asyncio.AbstractEventLoop,
    api_key: str,
    chat_uuid: str,
    settings: Settings,
    prompt: str | None,
    workflow_id: str | None,
    timeout_seconds: int | None,
) -> REMOTE_EXECUTION_CLIENT_EXIT_INFO | None:
    start_ts = time.time()
    base_url = settings.base_url
    base_api_url = settings.get_base_api_url()
    base_ws_url = settings.get_base_ws_url()

    print_exponent_message(base_url, chat_uuid)
    click.echo()

    connection_tracker = ConnectionTracker()

    client_fut = loop.create_task(
        start_client(
            api_key,
            base_url,
            base_api_url,
            base_ws_url,
            chat_uuid,
            prompt=prompt,
            workflow_id=workflow_id,
            connection_tracker=connection_tracker,
            timeout_seconds=timeout_seconds,
        )
    )

    conn_fut = loop.create_task(handle_connection_changes(connection_tracker, start_ts))

    try:
        done, _ = loop.run_until_complete(
            asyncio.wait({client_fut, conn_fut}, return_when=asyncio.FIRST_COMPLETED)
        )

        if client_fut in done:
            return client_fut.result()
        else:
            return None
    finally:
        for task in asyncio.all_tasks(loop):
            task.cancel()

        try:
            loop.run_until_complete(asyncio.wait(asyncio.all_tasks(loop)))
        except asyncio.CancelledError:
            pass


async def handle_connection_changes(
    connection_tracker: ConnectionTracker, start_ts: float
) -> None:
    try:
        async with timeout(5):
            assert await connection_tracker.next_change()
            click.echo(ready_message(start_ts))
    except TimeoutError:
        spinner = Spinner("Connecting...")
        spinner.show()
        assert await connection_tracker.next_change()
        spinner.hide()
        click.echo(ready_message(start_ts))

    while True:
        assert not await connection_tracker.next_change()

        click.echo("Disconnected...", nl=False)
        await asyncio.sleep(1)
        spinner = Spinner("Reconnecting...")
        spinner.show()
        assert await connection_tracker.next_change()
        spinner.hide()
        click.echo("\x1b[1;32m✓ Reconnected", nl=False)
        sys.stdout.flush()
        await asyncio.sleep(1)
        click.echo("\r\x1b[0m\x1b[2K", nl=False)
        sys.stdout.flush()


def ready_message(start_ts: float) -> str:
    elapsed = round(time.time() - start_ts, 2)
    return f"\x1b[32m✓\x1b[0m Ready in {elapsed}s"



HALT_CHAT_STREAM_MUTATION: str = """
  mutation HaltChatStream($chatUuid: UUID!) {
    haltChatStream(chatUuid: $chatUuid) {
      __typename
    }
  }
"""


SET_LOGIN_COMPLETE_MUTATION: str = """
  mutation SetLoginComplete {
    setLoginComplete {
      __typename
      ... on User {
        userApiKey
      }
      ... on UnauthenticatedError {
        message
      }
    }
  }
"""


REFRESH_API_KEY_MUTATION = """
mutation RefreshApiKey {
    refreshApiKey {
        ... on User {
            userApiKey
        }
        ... on UnauthenticatedError {
            message
        }
    }
}
"""

START_CHAT_TURN_MUTATION = """
mutation StartChatTurnMutation($chatInput: ChatInput!, $parentUuid: String, $chatConfig: ChatConfig!) {
    startChatReply(
        chatInput: $chatInput,
        parentUuid: $parentUuid,
        chatConfig: $chatConfig
    ) {
      __typename
      ... on UnauthenticatedError {
          message
      }
      ... on ChatNotFoundError {
          message
      }
      ... on Chat {
          chatUuid
      }
  }
}
"""


CREATE_CLOUD_CHAT_MUTATION = """
mutation CreateCloudChat($configId: String!) {
  createCloudChat(cloudConfigUuid: $configId) {
    __typename
    ...on Chat {
      chatUuid
    }
    ...on CloudSessionError {
      message
    }
    ...on UnauthenticatedError {
      message
    }
  }
}
"""


CREATE_CLOUD_CHAT_FROM_REPOSITORY_MUTATION = """
mutation CreateCloudChatFromRepository($repositoryId: String!) {
  createCloudChat(repositoryId: $repositoryId) {
    __typename
    ...on Chat {
      chatUuid
    }
    ...on UnauthenticatedError {
      message
    }
    ...on ChatNotFoundError {
      message
    }
    ...on CloudConfigNotFoundError {
      message
    }
    ...on GithubConfigNotFoundError {
      message
    }
    ...on CloudSessionError {
      message
    }
  }
}
"""


ENABLE_CLOUD_REPOSITORY_MUTATION = """
mutation EnableCloudRepository($orgName: String!, $repoName: String!) {
  enableCloudRepository(orgName: $orgName, repoName: $repoName) {
    __typename
    ...on ContainerImage {
      buildRef
      createdAt
      updatedAt
    }
    ...on UnauthenticatedError {
      message
    }
    ...on CloudConfigNotFoundError {
      message
    }
    ...on GithubConfigNotFoundError {
      message
    }
    ...on CloudSessionError {
      message
    }
    ...on Error {
      message
    }
  }
}
"""


INCREMENTAL_BUILD_CLOUD_REPOSITORY_MUTATION = """
mutation IncrementalBuildCloudRepository($orgName: String!, $repoName: String!) {
  incrementalBuildCloudRepository(orgName: $orgName, repoName: $repoName) {
    __typename
    ...on ContainerImage {
      buildRef
      createdAt
      updatedAt
    }
    ...on UnauthenticatedError {
      message
    }
    ...on CloudConfigNotFoundError {
      message
    }
    ...on GithubConfigNotFoundError {
      message
    }
    ...on CloudSessionError {
      message
    }
    ...on Error {
      message
    }
  }
}
"""


REBUILD_CLOUD_REPOSITORY_MUTATION = """
mutation RebuildCloudRepository($orgName: String!, $repoName: String!) {
  rebuildCloudRepository(orgName: $orgName, repoName: $repoName) {
    __typename
    ...on ContainerImage {
      buildRef
      createdAt
      updatedAt
    }
    ...on UnauthenticatedError {
      message
    }
    ...on CloudConfigNotFoundError {
      message
    }
    ...on GithubConfigNotFoundError {
      message
    }
    ...on CloudSessionError {
      message
    }
    ...on Error {
      message
    }
  }
}
"""



GET_CHATS_QUERY: str = """
    query Chats {
        chats {
            ... on UnauthenticatedError {
                message
            }
            ... on Chats {
                chats {
                    id
                    chatUuid
                    name
                    subtitle
                    isShared
                    isStarted
                    updatedAt # ISO datetime string
                }
            }
        }
    }
"""
CREATE_CLOUD_CONFIG_MUTATION: str = """
    mutation CreateCloudConfig(
        $githubOrgName: String!,
        $githubRepoName: String!,
        $setupCommands: [String!],
    ) {
        createCloudConfig(
            input: {
                githubOrgName: $githubOrgName,
                githubRepoName: $githubRepoName,
                setupCommands: $setupCommands,
            }
        ) {
            __typename
            ... on UnauthenticatedError {
                message
            }
            ... on CloudConfig {
                cloudConfigUuid
                githubOrgName
                githubRepoName
                setupCommands
                repoUrl
            }
        }
    }
"""



import getpass
import os
import platform

from exponent.core.remote_execution.git import get_git_info
from exponent.core.remote_execution.languages import python_execution
from exponent.core.remote_execution.types import (
    SystemInfo,
)


async def get_system_info(working_directory: str) -> SystemInfo:
    return SystemInfo(
        name=getpass.getuser(),
        cwd=working_directory,
        os=platform.system(),
        shell=_get_user_shell(),
        git=await get_git_info(working_directory),
        python_env=python_execution.get_python_env_info(),
    )


def _get_user_shell() -> str:
    return os.environ.get("SHELL", "bash")



import click
from click import Context, HelpFormatter

from exponent.commands.cloud_commands import cloud_cli
from exponent.commands.common import set_log_level
from exponent.commands.config_commands import config_cli
from exponent.commands.run_commands import run, run_cli
from exponent.commands.types import ExponentGroup, exponent_cli_group
from exponent.commands.upgrade import upgrade_cli
from exponent.commands.workflow_commands import workflow_cli
from exponent.utils.version import (
    get_installed_version,
)


@exponent_cli_group(invoke_without_command=True)
@click.version_option(get_installed_version(), prog_name="Indent CLI")
@click.pass_context
def cli(ctx: Context) -> None:
    """
    Indent: Your AI pair programmer.

    Run indent to start (or indent run)
    """
    set_log_level()

    # If no command is provided, invoke the 'run' command
    if ctx.invoked_subcommand is None:
        ctx.invoke(run)


sources: list[ExponentGroup] = [
    config_cli,  # Configuration commands
    run_cli,  # Run AI chat commands
    upgrade_cli,  # Upgrade-related commands
    cloud_cli,  # Cloud commands
    workflow_cli,  # Workflow commands
]

for source in sources:
    for command in source.commands.values():
        cli.add_command(command)


def format_commands(
    ctx: Context, formatter: HelpFormatter, include_hidden: bool = False
) -> None:
    commands = []
    hidden_commands = []
    for cmd_name in cli.list_commands(ctx):
        cmd = cli.get_command(ctx, cmd_name)
        if cmd is None:
            continue
        if cmd.hidden:
            hidden_commands.append((cmd_name, cmd))
        else:
            commands.append((cmd_name, cmd))

    if commands:
        max_cmd_length = (
            max(len(cmd_name) for cmd_name, _ in commands) if commands else 0
        )
        limit = (
            (formatter.width or 80) - 6 - max_cmd_length
        )  # Default width to 80 if None
        rows = []
        for cmd_name, cmd in commands:
            help_text = cmd.get_short_help_str(limit)
            rows.append((cmd_name, help_text))

        with formatter.section("Commands"):
            formatter.write_dl(rows)

    if include_hidden and hidden_commands:
        max_cmd_length = (
            max(len(cmd_name) for cmd_name, _ in hidden_commands)
            if hidden_commands
            else 0
        )
        limit = (
            (formatter.width or 80) - 6 - max_cmd_length
        )  # Default width to 80 if None
        hidden_rows = []
        for cmd_name, cmd in hidden_commands:
            help_text = cmd.get_short_help_str(limit)
            hidden_rows.append((cmd_name, help_text))

        with formatter.section("Hidden Commands"):
            formatter.write_dl(hidden_rows)


@cli.command(hidden=True)
@click.pass_context
def hidden(ctx: Context) -> None:
    """Show all commands, including hidden ones."""
    formatter = ctx.make_formatter()
    with formatter.section("Usage"):
        if ctx.parent and ctx.parent.command:
            formatter.write_usage(
                ctx.parent.command.name or "indent", "COMMAND [ARGS]..."
            )
    formatter.write_paragraph()
    with formatter.indentation():
        if cli.help:
            formatter.write_text(cli.help)
    formatter.write_paragraph()
    format_commands(ctx, formatter, include_hidden=True)
    click.echo(formatter.getvalue().rstrip("\n"))


if __name__ == "__main__":
    cli()



import asyncio
import os
import sys
from typing import cast

import click

from exponent.commands.run_commands import run_chat
from exponent.commands.settings import use_settings
from exponent.commands.types import exponent_cli_group
from exponent.core.config import Settings
from exponent.core.remote_execution.client import RemoteExecutionClient, WSDisconnected
from exponent.core.remote_execution.types import (
    PrReviewWorkflowInput,
    WorkflowTriggerResponse,
)


@exponent_cli_group(name="workflow")
def workflow_cli() -> None:
    """Workflow commands."""
    pass


@workflow_cli.group(hidden=True)
def workflow() -> None:
    """Workflow management commands."""
    pass


@workflow.command()
@use_settings
@click.argument("workflow_type", type=click.STRING)
def trigger(settings: Settings, workflow_type: str) -> None:
    """Trigger a workflow."""

    if not settings.api_key:
        raise click.ClickException(
            "No API key found. Use `indent login` to set your API key."
        )

    if workflow_type != "pr_review":
        raise click.UsageError("Invalid workflow name. Only 'pr_review' is supported.")

    loop = asyncio.get_event_loop()
    response = loop.run_until_complete(trigger_pr_review_workflow(settings))

    while True:
        result = run_chat(
            loop, settings.api_key, response.chat_uuid, settings, None, None, None
        )
        if result is None or isinstance(result, WSDisconnected):
            # NOTE: None here means that handle_connection_changes exited
            # first. We should likely have a different message for this.
            if result and result.error_message:
                click.secho(f"Error: {result.error_message}", fg="red")
                sys.exit(10)
            else:
                click.echo("Disconnected upon user request, shutting down...")
                break
        else:
            raise click.ClickException("Workflow run exited unexpectedly")


async def _subprocess_check_output(command: str) -> str:
    process = await asyncio.create_subprocess_shell(
        command,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.STDOUT,
    )
    stdout, _ = await process.communicate()

    if process.returncode != 0:
        output = stdout.decode().strip()
        raise click.ClickException(
            f"Command '{command}' failed with exit code {process.returncode}:\n{output}"
        )

    return stdout.decode().strip()


async def trigger_pr_review_workflow(settings: Settings) -> WorkflowTriggerResponse:
    origin_url = await _subprocess_check_output("git ls-remote --get-url origin")
    url = origin_url.strip().removesuffix(".git")
    remote = url.split(":")[-1]
    owner, repo = remote.split("/")[-2:]

    pr_number_str = os.environ.get("PR_NUMBER")
    if not pr_number_str:
        raise click.ClickException("PR_NUMBER environment variable is not set")
    try:
        pr_number = int(pr_number_str)
    except ValueError:
        raise click.ClickException(
            "PR_NUMBER environment variable is not a valid integer"
        )

    async with RemoteExecutionClient.session(
        api_key=cast(str, settings.api_key),
        base_url=settings.get_base_api_url(),
        base_ws_url=settings.get_base_ws_url(),
        working_directory=os.getcwd(),
    ) as client:
        return await client.trigger_workflow(
            workflow_name="pr_review",
            workflow_input=PrReviewWorkflowInput(
                repo_owner=owner,
                repo_name=repo,
                pr_number=pr_number,
            ),
        )
